-- Puffin: A Tojaso FowlPlay Project
-- Widget component to animate icons in other widgets
-- Supported animation styles: fade, delay, pulse, splash

-- Depends on attributes in the widget that holds the icon:
-- _display.container - icons's background frame
-- _display.texture - icons's texture
-- color - color to apply to icon (default: none)
-- alpha - opacity of the icon (default: 1)

local Widget_Animate = {}
local animationPool = {} -- pool of available animations
local animations = {} -- active animations

-- Start an animation for a widget's icon texture
function Widget_Animate.IconAnimation(widget, style)
	local t = widget._display
	local tex = t.texture:GetTexture(); if not tex then return end
	local ULx, ULy, LLx, LLy, URx, URy, LRx, LRy = t.texture:GetTexCoord()
	local offsetX, offsetY = t.container:GetLeft(), t.container:GetBottom()
	local b = next(animationPool)
	if b then animationPool[b] = nil else
		b = {} -- initializae a new animation
		b.frame = CreateFrame("Frame", nil, UIParent)
		b.frame:SetFrameLevel(t.container:GetFrameLevel() + 9)
		b.texture = b.frame:CreateTexture(nil, "ARTWORK") -- texture for the texture to be animated	
		b.anim = b.frame:CreateAnimationGroup()
		b.anim:SetLooping("NONE")
		b.grow = b.anim:CreateAnimation("Scale"); b.grow:SetOrigin('CENTER', 0, 0); b.grow:SetOrder(1)
		b.shrink = b.anim:CreateAnimation("Scale"); b.shrink:SetOrigin('CENTER', 0, 0); b.shrink:SetOrder(2)
		b.alpha = b.anim:CreateAnimation("Alpha")
	end
	local w, h = t.texture:GetSize()
	b.frame:ClearAllPoints(); b.frame:SetPoint("BOTTOMLEFT", nil, "BOTTOMLEFT", offsetX, offsetY); b.frame:SetSize(w, h); b.frame:Show()
	b.texture:SetTexture(tex); b.texture:SetTexCoord(ULx, ULy, LLx, LLy, URx, URy, LRx, LRy); b.texture:SetAlpha(widget.alpha or 1)
	local c = widget.color; if c then b.texture:SetVertexColor(c.r, c.g, c.b) else b.texture:SetVertexColor(1, 1, 1) end
	b.texture:ClearAllPoints(); b.texture:SetAllPoints(b.frame); b.texture:Show()
	local animationTime = 1
	if style == "fade" then -- 0.65 second fade
		b.grow:SetScale(1, 1); b.grow:SetDuration(0.1) -- disable grow
		b.shrink:SetScale(1, 1); b.shrink:SetDuration(0.1) -- disable shrink
		b.alpha:SetChange(-1); b.alpha:SetDuration(0.65); b.alpha:SetSmoothing("IN"); b.alpha:SetEndDelay(5); b.alpha:SetOrder(1)
	elseif style == "pulse" then -- 0.5 second fade with scale grow/shrink
		b.grow:SetScale(3, 3); b.grow:SetDuration(0.25)
		b.shrink:SetScale(-3, -3); b.shrink:SetDuration(0.25)
		b.alpha:SetChange(-1); b.alpha:SetDuration(0.25); b.alpha:SetSmoothing("IN"); b.alpha:SetEndDelay(5); b.alpha:SetOrder(2)
	elseif style == "delay" then -- 2 second fade
		b.grow:SetScale(1, 1); b.grow:SetDuration(0.1) -- disable grow
		b.shrink:SetScale(1, 1); b.shrink:SetDuration(0.1) -- disable shrink
		b.alpha:SetChange(-1); b.alpha:SetDuration(2); b.alpha:SetSmoothing("IN"); b.alpha:SetEndDelay(5); b.alpha:SetOrder(1)
		animationTime = 3
	else -- must be "splash" 0.65 sec fade with scale flyout
		b.grow:SetScale(3, 3); b.grow:SetDuration(0.65)
		b.shrink:SetScale(1, 1); b.shrink:SetDuration(0.1) -- disable shrink
		b.alpha:SetChange(-1); b.alpha:SetDuration(0.65); b.alpha:SetSmoothing("IN"); b.alpha:SetEndDelay(5); b.alpha:SetOrder(1)
	end
	b.anim:Stop(); b.anim:Play()
	b.endTime = GetTime() + animationTime -- stop after one second
	table.insert(animations, b)
end

-- Update active animations, recycling when they are complete
function Widget_Animate:OnUpdate()
	local now = GetTime()
	for k, b in pairs(animations) do
		if now > b.endTime then
			b.anim:Pause(); animations[k] = nil; animationPool[b] = true
			b.frame:ClearAllPoints(); b.texture:ClearAllPoints(); b.frame:Hide(); b.texture:Hide()
		end
	end
end

-- Register this component so it can be found by views
Puffin.RegisterComponent("Widget_Animate", Widget_Animate, "WIDGET", 0)
